#!/usr/bin/perl

use Time::Local;
use MIME::Base64;
use SOAP::Lite +trace => 'debug';
use SOAP::Lite;
my $username = 'YourUsername';
my $password = 'YourPassword';
my $message = 'Hello World!';
my @recipients = ('46701xxxxxx');
my $sender = 'Lekab';
my $endpoint = 'https://secure.infowireless.com/ws/messaging-v2';

sub xml_quote {
    my ($value) = @_;
    $value =~ s/&/&amp;/;
    $value =~ s/</&lt;/;
    $value =~ s/>/&gt;/;
    $value;
}

sub _complex_type {
    my ($name,@children) = @_;
    my $data = SOAP::Data->new( name => $name );
    $data->value( \SOAP::Data->value(@children));
    $data;
}

sub _typeless {
    my ($name,$value) = @_;
    my $data = SOAP::Data->new( name => $name );

    $value = xml_quote($value);

    $data->value( $value );
    $data->type( "" );
    $data;
}

sub created {
    my ($sec,$min,$hour,$mday,$mon,$year,undef,undef,undef) = gmtime(time);
    $mon++;
    $year = $year + 1900;
    return sprintf("%04d-%02d-%02dT%02d:%02d:%02dZ",$year,$mon,$mday,$hour,$min,$sec);
}

sub expires {
    my ($sec,$min,$hour,$mday,$mon,$year,undef,undef,undef) = gmtime(time + 600);
    $mon++;
    $year = $year + 1900;
    return sprintf("%04d-%02d-%02dT%02d:%02d:%02dZ",$year,$mon,$mday,$hour,$min,$sec);
}

sub ws_authen {
    my($username,$password) = @_;

    my $auth = SOAP::Header->new( name => "wsse:Security" );
    $auth->attr( {
        "xmlns:wsse" => "http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd",
        "xmlns:wsu"  => "http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-utility-1.0.xsd",
    });
    $auth->mustUnderstand(1);

    $auth->value( \SOAP::Data->value(
         _complex_type ( "wsse:UsernameToken",
            _typeless("wsse:Username",$username),
            _typeless("wsse:Password",$password)->attr({
                "Type" => "http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-username-token-profile-1.0#PasswordText"
            })),
         _complex_type ( "wsu:Timestamp",
            _typeless("wsu:Created",created()),
            _typeless("wsu:Expires",expires()))
        )
    );
    $auth;
}

my $soap = SOAP::Lite->new( proxy => $endpoint);
$soap->on_action( sub { "http://www.lekab.com/schema/messaging-v2/messages#SendRequest" });
$soap->autotype(0);
$soap->default_ns('http://www.lekab.com/schema/messaging-v2/messages');

my @sms = ( 
	SOAP::Data->name('sender')->value($sender),
        SOAP::Data->name('recipients' =>
                \SOAP::Data->value(SOAP::Data->name('recipient' => @recipients))),
        SOAP::Data->name('data')->value(
		\SOAP::Data->name('sms')->value(
			\SOAP::Data->name('payload')->value(
				\SOAP::Data->name('message')->value(encode_base64($message))
							   )
                                               )
                                       )
          ); 
 
my $result = $soap->call("SendRequest", @sms, ws_authen($username,$password));

unless ($result->fault) {
    print "\n";
    my $count = 0;
    for my $status ($result->valueof('//SendResponse/messageStatus')) {
        print "MessageStatus [" . $count++ . "]:\n"; 
        print "\tstatusCode: " . $status->{statusCode} . "\n";
        print "\tstatusText: " . $status->{statusText} . "\n";
        print "\tid:         " . $status->{id} . "\n";
        print "\tsender:     " . $status->{sender} . "\n";
        print "\trecipient:  " . $status->{recipient} . "\n";
        print "\ttime:       " . $status->{time} . "\n";
        print "\n";
    }
    print "\n";
} else {
    print "\n";
    print join ', ', 
    $result->faultcode, 
    $result->faultstring;
    print "\n\n";

    my $details = $result->faultdetail->{errorDetails}; 
    while ( (my $key, my $value) = each (%$details)) {
        if (ref($value) eq 'ARRAY') {
            my $count = 0;
            for my $detail (@$value) {
                print "ErrorDetail [" . $count++ . "]:\n";
                print "\terrorCode:        " . $detail->{errorCode} . "\n";
                print "\terrorDescription: " . $detail->{errorDescription} . "\n";
                print "\n";
            }
        } elsif (ref($value) eq 'HASH') {
                print "ErrorDetail:\n";
                print "\terrorCode:        " . $value->{errorCode} . "\n";
                print "\terrorDescription: " . $value->{errorDescription} . "\n";
                print "\n";
        }
    }
    print "\n";
}
